
from unittest.mock import patch
from odoo.addons.base.tests.common import HttpCaseWithUserPortal


class HttpCaseWithUserPortalAndWirePayment(HttpCaseWithUserPortal):

    def _setup_payment_transfer_for_test(self, set_done=False):
        if self.env['ir.module.module']._get('payment_custom').state != 'installed':
            self.skipTest("Transfer provider is not installed")

        transfer_provider = self.env.ref('payment.payment_provider_transfer')
        transfer_provider.write({
            'state': 'test',
            'is_published': True,
        })
        transfer_provider._transfer_ensure_pending_msg_is_set()

        if set_done:
            # force wire payment transactions to be set to "Done"
            tx_class = type(self.env["payment.transaction"])
            _orig_process_notification_data = tx_class._process_notification_data

            def _process_notification_data(self, notification_data):
                _orig_process_notification_data(self, notification_data)
                if self.provider_code == 'custom':
                    self._set_done()

            self.startPatcher(
                patch.object(tx_class, "_process_notification_data",
                             side_effect=_process_notification_data, autospec=True)
            )
            # no payment journal available, use a dummy _reconcile_after_done()
            # (this will still allow to auto-confirm sale.order but will not
            # reconcile the invoice)
            reconcile_after_done_patcher = patch(
                'odoo.addons.account_payment.models.payment_transaction.PaymentTransaction._reconcile_after_done',
            )
            self.startPatcher(reconcile_after_done_patcher)

    def _register_payment(self, invoice):
        """Fully pay the provided invoice."""
        PaymentRegister = self.env['account.payment.register']
        payment_context = {'active_model': 'account.move', 'active_ids': invoice.ids}
        payments = PaymentRegister.with_context(**payment_context).create({})._create_payments()
        return payments
